@@ -1,17 +1,21 @@
+from functools import partial
 from logging import getLogger
-logger = getLogger(__name__)
 
-from django.core.paginator import Paginator
-from django.shortcuts import render
-from django.utils.datastructures import SortedDict
 from django.conf import settings
-from django.utils.functional import curry
+from django.core.paginator import Paginator
 from django.http import HttpResponseNotFound
-
+from django.shortcuts import render
 from redis.exceptions import ResponseError
 
+from .utils import PY3
 from .utils import LazySlicingIterable
 
+try:
+    from django.utils.datastructures import SortedDict as OrderedDict
+except ImportError:
+    from collections import OrderedDict
+logger = getLogger(__name__)
+
 REDISBOARD_ITEMS_PER_PAGE = getattr(settings, 'REDISBOARD_ITEMS_PER_PAGE', 100)
 
 
@@ -36,43 +40,88 @@ LENGTH_GETTERS = {
 }
 
 
+def _decode_bytes(value):
+    if isinstance(value, bytes):
+        return value.decode('utf8')
+    else:
+        return value
+
+
 def _get_key_info(conn, key):
     try:
         obj_type = conn.type(key)
+        length_getter = LENGTH_GETTERS.get(obj_type)
+        if not length_getter:
+            return {
+                'type': 'none',
+                'name': key,
+                'length': "n/a",
+                'error': "The key does not exist",
+                'ttl': "n/a",
+                'refcount': "n/a",
+                'encoding': "n/a",
+                'idletime': "n/a",
+            }
+
         pipe = conn.pipeline()
-        pipe.execute_command('DEBUG', 'OBJECT', key)
-        LENGTH_GETTERS[obj_type](pipe, key)
-        pipe.ttl(key)
-        details, obj_length, obj_ttl = pipe.execute()
+
+        try:
+            pipe.object('REFCOUNT', key)
+            pipe.object('ENCODING', key)
+            pipe.object('IDLETIME', key)
+            length_getter(pipe, key)
+            pipe.ttl(key)
+
+            refcount, encoding, idletime, obj_length, obj_ttl = pipe.execute()
+        except ResponseError as exc:
+            logger.exception("Failed to get object info for key %r: %s", key, exc)
+            return {
+                'type': obj_type,
+                'name': key,
+                'length': "n/a",
+                'error': str(exc),
+                'ttl': "n/a",
+                'refcount': "n/a",
+                'encoding': "n/a",
+                'idletime': "n/a",
+            }
         return {
-            'type': obj_type,
+            'type': _decode_bytes(obj_type),
             'name': key,
-            'details': details if isinstance(details, dict) else dict(
-                _fixup_pair(i.split(b':')) for i in details.split() if b':' in i
-            ),
             'length': obj_length,
             'ttl': obj_ttl,
+            'refcount': refcount,
+            'encoding': _decode_bytes(encoding),
+            'idletime': idletime,
         }
     except ResponseError as exc:
-        logger.exception("Failed to get details for key %r", key)
+        logger.exception("Failed to get details for key %r: %s", key, exc)
         return {
             'type': "n/a",
             'length': "n/a",
             'name': key,
-            'details': {},
             'error': str(exc),
             'ttl': "n/a",
+            'refcount': "n/a",
+            'encoding': "n/a",
+            'idletime': "n/a",
         }
 
+
 VALUE_GETTERS = {
-    b'list': lambda conn, key, start=0, end=-1: [(pos + start, val)
-                                                 for (pos, val) in enumerate(conn.lrange(key, start, end))],
-    b'string': lambda conn, key, *args: [('string', conn.get(key))],
-    b'set': lambda conn, key, *args: list(enumerate(conn.smembers(key))),
-    b'zset': lambda conn, key, start=0, end=-1: [(pos + start, val)
-                                                 for (pos, val) in enumerate(conn.zrange(key, start, end))],
-    b'hash': lambda conn, key, *args: conn.hgetall(key).items(),
-    b'n/a': lambda conn, key, *args: (),
+    'list': lambda conn, key, start=0, end=-1: [
+        (pos + start, val)
+        for (pos, val) in enumerate(conn.lrange(key, start, end))
+    ],
+    'string': lambda conn, key, *args: [('string', conn.get(key))],
+    'set': lambda conn, key, *args: list(enumerate(conn.smembers(key))),
+    'zset': lambda conn, key, start=0, end=-1: [
+        (pos + start, val)
+        for (pos, val) in enumerate(conn.zrange(key, start, end))
+    ],
+    'hash': lambda conn, key, *args: conn.hgetall(key).items(),
+    'n/a': lambda conn, key, *args: (),
+    'none': lambda conn, key, *args: (),
 }
 
 
@@ -84,7 +133,7 @@ def _get_key_details(conn, db, key, page):
         details['data'] = Paginator(
             LazySlicingIterable(
                 lambda: details['length'],
-                curry(VALUE_GETTERS[details['type']], conn, key)
+                partial(VALUE_GETTERS[details['type']], conn, key)
             ),
             REDISBOARD_ITEMS_PER_PAGE
         ).page(page)
@@ -94,7 +143,7 @@ def _get_key_details(conn, db, key, page):
     return details
 
 
-def _get_db_summary(server, db):
+def _raw_get_db_summary(server, db):
     server.connection.execute_command('SELECT', db)
     pipe = server.connection.pipeline()
 
@@ -148,7 +197,6 @@ def _get_db_summary(server, db):
         volatile_memory = (volatile_memory / volatile_keys) * size
     else:
         volatile_memory = 0
-
     return dict(
         size=size,
         total_memory=total_memory,
@@ -157,6 +205,19 @@ def _get_db_summary(server, db):
     )
 
 
+def _get_db_summary(server, db):
+    try:
+        return _raw_get_db_summary(server, db)
+    except ResponseError as exc:
+        logger.exception("Failed to get summary for db %r: %s", db, exc)
+        return dict(
+            size=0,
+            total_memory=0,
+            volatile_memory=0,
+            persistent_memory=0,
+        )
+
+
 def _get_db_details(server, db):
     conn = server.connection
     conn.execute_command('SELECT', db)
@@ -166,7 +227,7 @@ def _get_db_details(server, db):
     if size > server.sampling_threshold:
         sampling = True
         pipe = conn.pipeline()
-        for _ in xrange(server.sampling_size):
+        for _ in (range if PY3 else xrange)(server.sampling_size):  # noqa
             pipe.randomkey()
 
         for key in set(pipe.execute()):
@@ -186,7 +247,7 @@ def _get_db_details(server, db):
 def inspect(request, server):
     stats = server.stats
     conn = server.connection
-    database_details = SortedDict()
+    database_details = OrderedDict()
     key_details = None
 
     if stats['status'] == 'UP':
